//--------------------------------------------------------------
// File     : stm32_ub_pwm_tim4.c
// Datum    : 17.01.2016
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F746
// IDE      : OpenSTM32
// GCC      : 4.9 2015q2
// Module   : CubeHAL
// Funktion : PWM-Funktionen per Timer4
//
// Hinweis  : mgliche Pinbelegungen
//            CH1 : [PB6, PD12]
//            CH2 : [PB7, PD13]
//            CH3 : [PB8, PD14]
//            CH4 : [PB9, PD15]
//--------------------------------------------------------------

//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_pwm_tim4.h"


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_PWM_InitIO(void);
void P_PWM_InitTIM(void);


//--------------------------------------------------------------
// Definition aller PWM Pins
// Reihenfolge wie bei PWM_TIM4_NAME_t
//
// Channel : [1...4]
// Init = Startwert : [0...PWM_TIM4_PERIODE]
//--------------------------------------------------------------
PWM_TIM4_t PWM_TIM4[] = {
  // Name     ,Channel, PORT , PIN       , Init
  {PWM_T4_PB8 ,3      ,GPIOB ,GPIO_PIN_8 ,  64},
  {PWM_T4_PB9 ,4      ,GPIOB ,GPIO_PIN_9 ,  128},
};


//--------------------------------------------------------------
// init und start vom PWM mit Timer4
//-------------------------------------------------------------- 
void UB_PWM_TIM4_Init(void)
{
  P_PWM_InitIO();
  P_PWM_InitTIM();
}


//--------------------------------------------------------------
// stellt einen PWM-Wert fr einen Kanal ein
// wert : [0 bis PWM_TIM4_PERIODE]
//--------------------------------------------------------------
void UB_PWM_TIM4_SetPWM(PWM_TIM4_NAME_t pwm, uint16_t wert)
{
  if(wert>PWM_TIM4_PERIODE) wert=PWM_TIM4_PERIODE;

  if(PWM_TIM4[pwm].CHANNEL==1) {
    TIM4->CCR1 = wert;
  }
  if(PWM_TIM4[pwm].CHANNEL==2) {
    TIM4->CCR2 = wert;
  }
  if(PWM_TIM4[pwm].CHANNEL==3) {
    TIM4->CCR3 = wert;
  }
  if(PWM_TIM4[pwm].CHANNEL==4) {
    TIM4->CCR4 = wert;
  }
}

//--------------------------------------------------------------
// interne Funktion
// Init aller IO-Pins
//--------------------------------------------------------------
void P_PWM_InitIO(void)
{
  GPIO_InitTypeDef GPIO_InitStruct;
  PWM_TIM4_NAME_t pwm_name;

  for(pwm_name=0;pwm_name<PWM_TIM4_ANZ;pwm_name++) {
    // Clock Enable
    UB_System_ClockEnable(PWM_TIM4[pwm_name].PWM_PORT);

    // Config des Pins als Digital-Ausgang
    GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
    GPIO_InitStruct.Pull = GPIO_PULLUP;
    GPIO_InitStruct.Speed = GPIO_SPEED_HIGH;
    GPIO_InitStruct.Alternate = GPIO_AF1_TIM4;
    GPIO_InitStruct.Pin = PWM_TIM4[pwm_name].PWM_PIN;
    HAL_GPIO_Init(PWM_TIM4[pwm_name].PWM_PORT, &GPIO_InitStruct);
  }  
}

//--------------------------------------------------------------
// interne Funktion
// Init vom Timer
//--------------------------------------------------------------
void P_PWM_InitTIM(void)
{
  TIM_HandleTypeDef TimHandle;
  TIM_OC_InitTypeDef sConfig;
  PWM_TIM4_NAME_t pwm_name;
  uint16_t wert;

  // Clock enable
  __HAL_RCC_TIM4_CLK_ENABLE();

  // Timer init
  TimHandle.Instance = TIM4;
  TimHandle.Init.Prescaler         = PWM_TIM4_PRESCALE;
  TimHandle.Init.Period            = PWM_TIM4_PERIODE;
  TimHandle.Init.ClockDivision     = TIM_CLOCKDIVISION_DIV1;
  TimHandle.Init.CounterMode       = TIM_COUNTERMODE_UP;
  TimHandle.Init.RepetitionCounter = 0;
  HAL_TIM_PWM_Init(&TimHandle);

  for(pwm_name=0;pwm_name<PWM_TIM4_ANZ;pwm_name++) {
    if(PWM_TIM4[pwm_name].CHANNEL==1) {
      // Channel 1
      sConfig.OCMode       = TIM_OCMODE_PWM1;
      sConfig.OCPolarity   = PWM_TIM4_POLARITY;
      sConfig.OCFastMode   = TIM_OCFAST_DISABLE;
      sConfig.OCNPolarity  = TIM_OCNPOLARITY_HIGH;
      sConfig.OCNIdleState = TIM_OCNIDLESTATE_RESET;
      sConfig.OCIdleState  = TIM_OCIDLESTATE_RESET;
      wert=PWM_TIM4[pwm_name].PWM_INIT;
      if(wert>PWM_TIM4_PERIODE) wert=PWM_TIM4_PERIODE;
      sConfig.Pulse = wert;
      HAL_TIM_PWM_ConfigChannel(&TimHandle, &sConfig, TIM_CHANNEL_1);
      HAL_TIM_PWM_Start(&TimHandle, TIM_CHANNEL_1);
    }
    if(PWM_TIM4[pwm_name].CHANNEL==2) {
      // Channel 2
      sConfig.OCMode       = TIM_OCMODE_PWM1;
      sConfig.OCPolarity   = PWM_TIM4_POLARITY;
      sConfig.OCFastMode   = TIM_OCFAST_DISABLE;
      sConfig.OCNPolarity  = TIM_OCNPOLARITY_HIGH;
      sConfig.OCNIdleState = TIM_OCNIDLESTATE_RESET;
      sConfig.OCIdleState  = TIM_OCIDLESTATE_RESET;
      wert=PWM_TIM4[pwm_name].PWM_INIT;
      if(wert>PWM_TIM4_PERIODE) wert=PWM_TIM4_PERIODE;
      sConfig.Pulse = wert;
      HAL_TIM_PWM_ConfigChannel(&TimHandle, &sConfig, TIM_CHANNEL_2);
      HAL_TIM_PWM_Start(&TimHandle, TIM_CHANNEL_2);
    }
    if(PWM_TIM4[pwm_name].CHANNEL==3) {
      // Channel 3
      sConfig.OCMode       = TIM_OCMODE_PWM1;
      sConfig.OCPolarity   = PWM_TIM4_POLARITY;
      sConfig.OCFastMode   = TIM_OCFAST_DISABLE;
      sConfig.OCNPolarity  = TIM_OCNPOLARITY_HIGH;
      sConfig.OCNIdleState = TIM_OCNIDLESTATE_RESET;
      sConfig.OCIdleState  = TIM_OCIDLESTATE_RESET;
      wert=PWM_TIM4[pwm_name].PWM_INIT;
      if(wert>PWM_TIM4_PERIODE) wert=PWM_TIM4_PERIODE;
      sConfig.Pulse = wert;
      HAL_TIM_PWM_ConfigChannel(&TimHandle, &sConfig, TIM_CHANNEL_3);
      HAL_TIM_PWM_Start(&TimHandle, TIM_CHANNEL_3);
    }
    if(PWM_TIM4[pwm_name].CHANNEL==4) {
      // Channel 4
      sConfig.OCMode       = TIM_OCMODE_PWM1;
      sConfig.OCPolarity   = PWM_TIM4_POLARITY;
      sConfig.OCFastMode   = TIM_OCFAST_DISABLE;
      sConfig.OCNPolarity  = TIM_OCNPOLARITY_HIGH;
      sConfig.OCNIdleState = TIM_OCNIDLESTATE_RESET;
      sConfig.OCIdleState  = TIM_OCIDLESTATE_RESET;
      wert=PWM_TIM4[pwm_name].PWM_INIT;
      if(wert>PWM_TIM4_PERIODE) wert=PWM_TIM4_PERIODE;
      sConfig.Pulse = wert;
      HAL_TIM_PWM_ConfigChannel(&TimHandle, &sConfig, TIM_CHANNEL_4);
      HAL_TIM_PWM_Start(&TimHandle, TIM_CHANNEL_4);
    }
  }
}

